@@ -2,11 +2,10 @@ import pytest
 from _pytest.config import ExitCode
 
 
-def test_version(testdir, pytestconfig):
+def test_version_verbose(testdir, pytestconfig):
     testdir.monkeypatch.delenv("PYTEST_DISABLE_PLUGIN_AUTOLOAD")
-    result = testdir.runpytest("--version")
+    result = testdir.runpytest("--version", "--version")
     assert result.ret == 0
-    # p = py.path.local(py.__file__).dirpath()
     result.stderr.fnmatch_lines(
         ["*pytest*{}*imported from*".format(pytest.__version__)]
     )
@@ -14,12 +13,23 @@ def test_version(testdir, pytestconfig):
         result.stderr.fnmatch_lines(["*setuptools registered plugins:", "*at*"])
 
 
+def test_version_less_verbose(testdir, pytestconfig):
+    testdir.monkeypatch.delenv("PYTEST_DISABLE_PLUGIN_AUTOLOAD")
+    result = testdir.runpytest("--version")
+    assert result.ret == 0
+    # p = py.path.local(py.__file__).dirpath()
+    result.stderr.fnmatch_lines(["pytest {}".format(pytest.__version__)])
+
+
 def test_help(testdir):
     result = testdir.runpytest("--help")
     assert result.ret == 0
     result.stdout.fnmatch_lines(
         """
-        *-v*verbose*
+          -m MARKEXPR           only run tests matching given mark expression.
+                                For example: -m 'mark1 and not mark2'.
+        reporting:
+          --durations=N *
         *setup.cfg*
         *minversion*
         *to see*markers*pytest --markers*
